<?php

class PeepsoMigrateAdminMigrate {

	var $limit = 0;
	var $next_limit = 0;
	var $per_page = 50;
	var $peepso_dir;
	var $counter = 0;

	function __construct() {
		$this->peepso_dir = PeepSo::get_peepso_dir();

		if (isset($_REQUEST["limit"])) {
			$this->limit = $_REQUEST["limit"];
		}

		$this->next_limit = $this->limit + $this->per_page;

		if (!function_exists('get_home_path')) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}
	}

	function startMigrateProfiles() {
		global $wpdb;

		// move gender field
		$gender_field = get_transient('peepso_migrate_gender_field');
		$gender_field_male = get_transient('peepso_migrate_gender_field_male');
		$gender_field_female = get_transient('peepso_migrate_gender_field_female');

		if (!empty($gender_field) && !empty($gender_field_male) && !empty($gender_field_female)) {

			$sql = $wpdb->prepare("SELECT A.user_id, C.id AS option_id FROM " . $wpdb->prefix . "bp_xprofile_data A " .
					"LEFT JOIN " . $wpdb->prefix . "users B ON B.ID = A.user_id " .
					"LEFT JOIN " . $wpdb->prefix . "bp_xprofile_fields C ON C.parent_id = A.field_id AND C.name = A.value " .
					"WHERE field_id = %d LIMIT %d, %d", $gender_field, $this->limit, $this->per_page);
			$all_gender_field = $wpdb->get_results($sql);

			if (isset($all_gender_field) && 0 < count($all_gender_field)) {

				$sql = $wpdb->prepare("SELECT A.* FROM " . $wpdb->prefix . "bp_xprofile_fields A " .
						"WHERE parent_id = %d AND type = %s AND id NOT IN ( %d, %d) ORDER BY id ASC", $gender_field, 'option', $gender_field_male, $gender_field_female);
				$all_custom_gender = $wpdb->get_results($sql);

				foreach ($all_gender_field as $key => $value) {

					if ($value->option_id == $gender_field_female) {
						$gender = 'f';
					} else if ($value->option_id == $gender_field_male) {
						$gender = 'm';
					} else {

						if (class_exists('PeepSoExtendedProfiles')) {

							for ($i = 1; $i <= count($all_custom_gender); $i++) {
								$gender_field_custom = get_transient('peepso_migrate_gender_field_custom' . ($i + 2));

								if (!empty($gender_field_custom) && ($gender_field_custom == $value->option_id)) {
									$gender = 'c' . ($gender_field_custom);
								}
							}
						} else {
							$gender = 'u';
						}
					}

					$sql = $wpdb->prepare("INSERT INTO " . $wpdb->prefix . "usermeta (`user_id`, `meta_key`, `meta_value`) values (%d, %s, %s)", $value->user_id, 'peepso_user_field_gender', $gender);
					$wpdb->query($sql);
				}
			}
		}

		// move birthdate field
		$birthdate_field = get_transient('peepso_migrate_birthdate_field');
		if (!empty($birthdate_field)) {
			$sql = $wpdb->prepare("SELECT A.* FROM " . $wpdb->prefix . "bp_xprofile_data A " .
					"LEFT JOIN " . $wpdb->prefix . "users B ON B.ID = A.user_id " .
					"WHERE field_id = %d LIMIT %d, %d", $birthdate_field, $this->limit, $this->per_page);
			$all_birthdate_field = $wpdb->get_results($sql);

			if (isset($all_birthdate_field) && 0 < count($all_birthdate_field)) {
				foreach ($all_birthdate_field as $key => $value) {
					$birthdate = str_replace(' 00:00:00', '', $value->value);
					$sql = $wpdb->prepare("INSERT INTO " . $wpdb->prefix . "usermeta (`user_id`, `meta_key`, `meta_value`) values (%d, %s, %s)", $value->user_id, 'peepso_user_field_birthdate', $birthdate);
					$wpdb->query($sql);
				}
			}
		}

		$sql = $wpdb->prepare("SELECT u.*, m.`meta_value` as last_activity FROM " . $wpdb->prefix . "users u left outer join " . $wpdb->prefix . "usermeta m on u.`ID`=m.`user_id` and m.`meta_key`='last_activity' limit %d, %d", $this->limit, $this->per_page);
		$all_users = $wpdb->get_results($sql);

		if (isset($all_users) && 0 < count($all_users)) {
			foreach ($all_users as $key => $user) {
				$sql = $wpdb->prepare("select count(*) as total from " . $wpdb->prefix . "peepso_users where `usr_id`=%d", intval($user->ID));
				$total = $wpdb->get_results($sql);
				$total = $total["0"]->total;

				if (0 == intval($total)) {

					$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_users (`usr_id`, `usr_last_activity`, `usr_views`, `usr_likes`, `usr_role`, `usr_send_emails`, `usr_cover_photo`, `usr_avatar_custom`, `usr_profile_acc`, `usr_first_name_acc`, `usr_last_name_acc`, `usr_description_acc`, `usr_user_url_acc`, `usr_gender`, `usr_gender_acc`, `usr_birthdate`, `usr_birthdate_acc`) values ('%d', '%s', '0', '0', 'member', '1', '', '0', '10', '10', '10', '10', '10', 'u', '10', '', '10')", intval($user->ID), trim($user->last_activity));
					$wpdb->query($sql);

					// set visibility for gender and birthdate
					$gender_acc = PeepSo::ACCESS_PUBLIC;
					if (!empty($gender_field) && !empty($gender_field_male) && !empty($gender_field_female)) {
						$gender_acc = $this->_get_field_visibility(intval($user->ID), $gender_field);
						update_user_meta(intval($user->ID), 'peepso_user_field_gender_acc', $gender_acc);
					}

					$birthdate_acc = PeepSo::ACCESS_PUBLIC;
					if (!empty($birthdate_field)) {
						$birthdate_acc = $this->_get_field_visibility(intval($user->ID), $birthdate_field);
						update_user_meta(intval($user->ID), 'peepso_user_field_birthdate_acc', $birthdate_acc);
					}
				}
			}
			$next_url = PeepsoMigrate::url('profile&limit=' . $this->next_limit);
		} else {
			$next_url = PeepsoMigrate::url('avatar');
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'profile',
			'counter' => count($all_users),
			'general' => 1,
		));
	}

	function startMigrateAvatars() {
		global $wpdb;

		$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "users limit %d, %d", $this->limit, $this->per_page);
		$all_users = $wpdb->get_results($sql);

		@mkdir($this->peepso_dir);
		$target_dir = $this->peepso_dir . 'users';
		@mkdir($target_dir);

		if (isset($all_users) && 0 < count($all_users)) {
			$path = get_home_path();

			foreach ($all_users as $key => $user) {
				@mkdir($target_dir . DIRECTORY_SEPARATOR . $user->ID);

				if (is_dir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "avatars" . DIRECTORY_SEPARATOR . $user->ID)) {
					$buddy_avatars = scandir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "avatars" . DIRECTORY_SEPARATOR . $user->ID);
					if (isset($buddy_avatars) && 0 < count($buddy_avatars)) {
						foreach ($buddy_avatars as $poz => $file) {
							if ("." != $file && ".." != $file && "" != $file) {
								if (FALSE !== strpos($file, "bpfull")) {
									$file_array = explode(".", $file);
									$extension = $file_array[count($file_array) - 1];

									$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "avatars" . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . $file;
									$destination1 = $target_dir . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . 'avatar-full.jpg';
									$destination2 = $target_dir . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . 'avatar-orig.jpg';

									if ($extension == 'png') {
										$this->_png2jpg($source, $destination1);
										$this->_png2jpg($source, $destination2);
									} else {
										copy($source, $destination1);
										copy($source, $destination2);
									}

									$sql = $wpdb->prepare("update " . $wpdb->prefix . "peepso_users set `usr_avatar_custom`='1' where `usr_id`='%d'", intval($user->ID));
									$wpdb->query($sql);
								} else if (FALSE !== strpos($file, "bpthumb")) {
									$file_array = explode(".", $file);
									$extension = $file_array[count($file_array) - 1];

									$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "avatars" . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . $file;
									$destination = $target_dir . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . 'avatar.jpg';

									if ($extension == 'png') {
										$this->_png2jpg($source, $destination);
									} else {
										copy($source, $destination);
									}

									$sql = $wpdb->prepare("update " . $wpdb->prefix . "peepso_users set `usr_avatar_custom`='1' where `usr_id`='%d'", intval($user->ID));
									$wpdb->query($sql);
								}
							}
						}
					}
				}

				if (is_dir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "members" . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . "cover-image")) {
					$buddy_cover = scandir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "members" . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . "cover-image");

					if (isset($buddy_cover) && 0 < count($buddy_cover)) {
						foreach ($buddy_cover as $poz => $file) {
							if ("." != $file && ".." != $file && "" != $file) {
								if (FALSE !== strpos($file, "bp-cover-image")) {
									$file_array = explode(".", $file);
									$extension = $file_array[count($file_array) - 1];

									$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "members" . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . "cover-image" . DIRECTORY_SEPARATOR . $file;
									$destination = $target_dir . DIRECTORY_SEPARATOR . $user->ID . DIRECTORY_SEPARATOR . 'cover.jpg';

									if ($extension == 'png') {
										$this->_png2jpg($source, $destination);
									} else {
										copy($source, $destination);
									}

									$site_url = get_site_url();
									$image_path = PeepSo::get_peepso_uri() . "users/" . $user->ID . "/cover.jpg";

									$sql = $wpdb->prepare("update " . $wpdb->prefix . "peepso_users set `usr_cover_photo`='%s' where `usr_id`='%d'", trim($image_path), intval($user->ID));
									$wpdb->query($sql);
								}
							}
						}
					}
				}
			}
			$next_url = PeepsoMigrate::url('avatar&limit=' . $this->next_limit);
		} else {
			$next_url = PeepsoMigrate::url('friend');
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'avatar',
			'counter' => count($all_users),
			'general' => 2,
		));
	}

	function startMigrateFriends() {
		$next_url = PeepsoMigrate::url('activity');

		if (PeepsoMigrate::plugin_check('friend')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_friends limit %d, %d", $this->limit, $this->per_page);
			$all_friends = $wpdb->get_results($sql);

			if (isset($all_friends) && 0 < count($all_friends)) {
				foreach ($all_friends as $key => $value) {
					$fnd_user_id = $value->initiator_user_id;
					$fnd_friend_id = $value->friend_user_id;
					$fnd_created = $value->date_created;
					$freq_viewed = $value->is_confirmed;

					if (0 == $freq_viewed) {
						$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_friend_requests (`freq_user_id`, `freq_friend_id`, `freq_created`, `freq_viewed`) values ('%d', '%d', '%s', '%d')", intval($fnd_user_id), intval($fnd_friend_id), $fnd_created, intval($freq_viewed));
						$wpdb->query($sql);
					} else {
						$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_friends (`fnd_user_id`, `fnd_friend_id`, `fnd_created`) values ('%d', '%d', '%s')", intval($fnd_user_id), intval($fnd_friend_id), $fnd_created);
						$wpdb->query($sql);
					}
				}
				$next_url = PeepsoMigrate::url('friend&limit=' . $this->next_limit);
			}

			$this->counter = count($all_friends);
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'friend',
			'counter' => $this->counter,
			'general' => 3,
		));
	}

	public function startMigrateActivity() {
		$next_url = PeepsoMigrate::url('notification');

		if (PeepsoMigrate::plugin_check('activity')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_activity WHERE `component` NOT IN ('members', 'groups', 'friends') limit %d, %d", $this->limit, $this->per_page);
			$all_activities = $wpdb->get_results($sql);

			if (isset($all_activities) && 0 < count($all_activities)) {
				foreach ($all_activities as $key => $post) {
					if ('members' == $post->component || 'groups' == $post->component) {
						continue;
					}

					// check if parent activity is group
					$is_group_activity = FALSE;

					$sql = "SELECT A.id, A.component, @pv:=A.secondary_item_id AS parent FROM (select * from " . $wpdb->prefix . "bp_activity ORDER BY id DESC) A JOIN (SELECT @pv:=$post->secondary_item_id) tmp WHERE A.id=@pv";
					$parents = $wpdb->get_results($sql);

					if (count($parents) > 0) {
						foreach ($parents as $p) {
							if ($p->component == 'groups') {
								$is_group_activity = TRUE;
							}
						}
					}

					if ($is_group_activity) {
						continue;
					}

					$post_type = 'peepso-post';

					if (0 != $post->secondary_item_id) {
						$post_type = 'peepso-comment';
					}

					$post_content = strip_tags($post->content);

					if (empty($post_content)) {
						continue;
					}

					// convert tags
					if (class_exists('PeepSoTags')) {
						preg_match_all("/\@(\w+) /", $post_content, $matches);
						if (count($matches[1]) > 0) {
							foreach ($matches[1] as $username) {
								$user = get_user_by('slug', $username);

								if ($user) {
									$post_content = str_replace('@' . $username, '[peepso_tag id=' . $user->ID . ']' . $username . '[/peepso_tag]', $post_content);
								}
							}
						}
					}

					$aPostData = array(
						'post_author' => $post->user_id,
						'post_date' => $post->date_recorded,
						'post_date_gmt' => $post->date_recorded,
						'post_content' => $post_content,
						'post_title' => $post->user_id . "-" . $post->user_id . "-" . time(),
						'post_excerpt' => $post_content,
						'post_status' => 'publish',
						'comment_status' => 'open',
						'ping_status' => 'open',
						'post_password' => '',
						'post_name' => $post->user_id . "-" . $post->user_id . "-" . time(),
						'to_ping' => '',
						'pinged' => '',
						'post_modified' => $post->date_recorded,
						'post_modified_gmt' => $post->date_recorded,
						'post_content_filtered' => '',
						'post_parent' => 0,
						'guid' => get_site_url() . '/' . $post_type . '/' . $post->user_id . "-" . $post->user_id . "-" . time() . '/',
						'menu_order' => 0,
						'post_type' => $post_type,
						'post_mime_type' => '',
						'comment_count' => 0
					);

					$peepso_new_post_id = wp_insert_post($aPostData);

					$this->add_memory('activity', $post->id, $peepso_new_post_id);

					if (0 != $post->secondary_item_id) {
						$bp_peepso_parent_id = $this->get_memory('activity', $post->item_id);

						// if empty, it's group post
						if (empty($bp_peepso_parent_id)) {
							continue;
						}

						// if regular comment
						if ($post->item_id == $post->secondary_item_id) {
							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '1', '%d', '1', '1')", $post->user_id, intval($peepso_new_post_id), $bp_peepso_parent_id);
							$wpdb->query($sql);
						} else {
							if (count($parents) > 0) {
								foreach ($parents as $p) {
									if ($p->parent == $post->item_id) {
										$bp_peepso_parent_id = $this->get_memory('activity', $p->id);
									}
								}
							}

							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '1', '%d', '1', '1')", $post->user_id, intval($peepso_new_post_id), $bp_peepso_parent_id);
							$wpdb->query($sql);
						}
					} else {
						$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '0', '0', '0', '1')", $post->user_id, intval($peepso_new_post_id));
						$wpdb->query($sql);
					}
				}
				$next_url = PeepsoMigrate::url('activity&limit=' . $this->next_limit);

				$this->counter = count($all_activities);
			}
		}


		echo json_encode(array(
			'url' => $next_url,
			'class' => 'activity',
			'counter' => $this->counter,
			'general' => 4,
		));
	}

	function startMigrateNotifications() {
		$next_url = PeepsoMigrate::url('message');

		if (PeepsoMigrate::plugin_check('notification')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT bpn.*, bpa.user_id as not_from_user_id FROM " . $wpdb->prefix . "bp_notifications bpn, " . $wpdb->prefix . "bp_activity bpa where bpn.`item_id`=bpa.`id` limit %d, %d", $this->limit, $this->per_page);
			$all_notifications = $wpdb->get_results($sql);

			if (isset($all_notifications) && 0 < count($all_notifications)) {
				foreach ($all_notifications as $key => $value) {
					$type = $msg = '';

					if ($value->component_name == 'activity' && $value->component_action == 'update_reply') {
						$type = 'user_comment';
						$msg = 'commented';
					}

					$not_external_id = $this->get_memory('activity', $value->item_id);

					if (!empty($type) && !empty($not_external_id)) {
						$data = array(
							'not_user_id' => $value->user_id,
							'not_from_user_id' => $value->not_from_user_id,
							'not_module_id' => 1,
							'not_external_id' => $not_external_id,
							'not_type' => $type,
							'not_message' => $msg,
							'not_timestamp' => $value->date_notified,
							'not_read' => $value->is_new
						);

						$wpdb->insert($wpdb->prefix . 'peepso_notifications', $data);
					}
				}
				$next_url = PeepsoMigrate::url('notification&limit=' . $this->next_limit);

				$this->counter = count($all_notifications);
			}
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'notification',
			'counter' => $this->counter,
			'general' => 5,
		));
	}

	function startMigrateMessages() {
		$next_url = PeepsoMigrate::url('profile_field');

		if (PeepsoMigrate::plugin_check('message')) {
			global $wpdb;
			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_messages_messages ORDER BY id ASC LIMIT %d, %d ", $this->limit, $this->per_page);
			$all_messages = $wpdb->get_results($sql);

			if (isset($all_messages) && 0 < count($all_messages)) {
				$thread_id = $prev_thread_id = 0;

				$existing_thread_id = $existing_parent_post_id = $existing_recipients = array();

				foreach ($all_messages as $key => $value) {
					$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_messages_recipients where `thread_id`=%d", intval($value->thread_id));
					$all_recipients = $wpdb->get_results($sql);

					$current_recipients = $unread_count = $is_deleted = array();

					if (isset($all_recipients) && 0 < count($all_recipients)) {
						foreach ($all_recipients as $rec_key => $rec_value) {
							$current_recipients[] = intval($rec_value->user_id);
							$unread_count[$value->thread_id . '-' . $rec_value->user_id] = $rec_value->unread_count;
							$is_deleted[$value->thread_id . '-' . $rec_value->user_id] = $rec_value->is_deleted;
						}
					}

					sort($current_recipients);

					$aPostData = array(
						'post_author' => $value->sender_id,
						'post_date' => $value->date_sent,
						'post_date_gmt' => $value->date_sent,
						'post_content' => strip_tags($value->message),
						'post_title' => '',
						'post_excerpt' => '',
						'post_status' => 'publish',
						'comment_status' => 'open',
						'ping_status' => 'open',
						'post_password' => '',
						'post_name' => '',
						'to_ping' => '',
						'pinged' => '',
						'post_modified' => $value->date_sent,
						'post_modified_gmt' => $value->date_sent,
						'post_content_filtered' => '',
						'post_parent' => 0,
						'guid' => '',
						'menu_order' => 0,
						'post_type' => 'peepso-message',
						'post_mime_type' => '',
						'comment_count' => 0
					);

					$post_id = wp_insert_post($aPostData);

					if ($value->thread_id != $prev_thread_id) {
						$prev_thread_id = $value->thread_id;

						if (in_array($current_recipients, $existing_recipients)) {
							// if same exact recipients means same conversation
							$parent_post_id = array_search($current_recipients, $existing_recipients);
						} else if (!in_array(intval($value->thread_id), $existing_thread_id)) {
							// if same thread id
							$parent_post_id = intval($post_id);
							$existing_thread_id[] = intval($value->thread_id);
							$existing_parent_post_id[intval($value->thread_id)] = $post_id;

							if (!in_array($current_recipients, $existing_recipients)) {
								$existing_recipients[$parent_post_id] = $current_recipients;
							}

							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_message_participants (`mpart_msg_id`, `mpart_user_id`) values ('%d', '%d')", $post_id, $value->sender_id);
							$wpdb->query($sql);

							foreach ($current_recipients as $recipient) {
								if ($recipient !== intval($value->sender_id)) {
									$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_message_participants (`mpart_msg_id`, `mpart_user_id`) values ('%d', '%d')", $post_id, $recipient);
									$wpdb->query($sql);
								}
							}

							reset($current_recipients);
						}
					}

					$sql = $wpdb->prepare("update " . $wpdb->prefix . "posts set `post_name`= %d, `post_parent` = %d, `guid`= %s where `ID`=%d", $post_id, $parent_post_id, get_site_url() . 'peepso-message/' . intval($post_id), intval($post_id));
					$wpdb->query($sql);

					// insert recipients
					$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_message_recipients (`mrec_msg_id`, `mrec_parent_id`, `mrec_user_id`, `mrec_viewed`, `mrec_deleted`) values (%d, %d, %d, %d, %d)", $post_id, $parent_post_id, $value->sender_id, $unread_count[$value->thread_id . '-' . $value->sender_id], $is_deleted[$value->thread_id . '-' . $value->sender_id]);
					$wpdb->query($sql);

					foreach ($current_recipients as $recipient) {
						if ($recipient !== intval($value->sender_id)) {
							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_message_recipients (`mrec_msg_id`, `mrec_parent_id`, `mrec_user_id`, `mrec_viewed`, `mrec_deleted`) values (%d, %d, %d, %d, %d)", $post_id, $parent_post_id, $recipient, $unread_count[$value->thread_id . '-' . $recipient], $is_deleted[$value->thread_id . '-' . $recipient]);
							$wpdb->query($sql);
						}
					}

					// insert activity
					$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_module_id`, `act_access`) values 
								(%d, %d, 6, 10)", $value->sender_id, $post_id);
					$wpdb->query($sql);
				}
				$next_url = PeepsoMigrate::url('message&limit=' . $this->next_limit);
			}

			$this->counter = count($all_messages);
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'message',
			'counter' => $this->counter,
			'general' => 6,
		));
	}

	function startMigrateProfileFields() {
		$next_url = PeepsoMigrate::url('group');

		// install core profile fields
		PeepSoProfileFields::install();
			
		if (PeepsoMigrate::plugin_check('profile_field')) {
			global $wpdb;

			$gender_field = get_transient('peepso_migrate_gender_field');
			$gender_field_male = get_transient('peepso_migrate_gender_field_male');
			$gender_field_female = get_transient('peepso_migrate_gender_field_female');

			$birthdate_field = get_transient('peepso_migrate_birthdate_field');

			// get all profile fields, exclude name, gender, and birthdate
			if (!empty($gender_field) && !empty($gender_field_male) && !empty($gender_field_female)) {
				$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_fields WHERE parent_id = 0 AND id NOT IN (%d, %d, %d) ORDER BY id, group_id LIMIT %d, %d", 1, intval($gender_field), intval($birthdate_field), $this->limit, $this->per_page);
			} else {

				$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_fields WHERE parent_id = 0 AND id NOT IN (%d, %d) ORDER BY id, group_id LIMIT %d, %d", 1, intval($birthdate_field), $this->limit, $this->per_page);
			}

			$all_profile_fields = $wpdb->get_results($sql);

			// set default field order to add
			$field_order = 6;
			$last_field_order = 0;
			$current_group_id = 1;

			if (isset($all_profile_fields) && 0 < count($all_profile_fields)) {
				foreach ($all_profile_fields as $key => $value) {

					// check if same group
					if ($current_group_id != $value->group_id) {
						$current_group_id = $value->group_id;

						// get field group data
						$field_group = $wpdb->get_row($wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_groups WHERE id = %d", $current_group_id));

						// add separator
						$aPostData = array(
							'post_author' => 1,
							'post_date' => current_time('mysql'),
							'post_date_gmt' => current_time('mysql'),
							'post_title' => $field_group->name,
							'post_content' => $field_group->name,
							'post_status' => 'publish',
							'comment_status' => 'closed',
							'ping_status' => 'closed',
							'post_parent' => 0,
							'guid' => get_bloginfo('url') . '/peepso_user_field/cpf/',
							'post_type' => 'peepso_user_field',
						);

						$post_id = wp_insert_post($aPostData);

						wp_update_post(array(
							'ID' => $post_id,
							'post_name' => $post_id
						));

						// postmeta
						$field_order = $last_field_order;
						$postmeta = array(
							'order' => $field_order,
							'class' => 'separator',
							'is_core' => 0,
							'default_title' => 1,
							'default_acc' => 10,
							'_wp_old_slug' => 'cpf',
							'method' => '_render',
							'method_form' => '_render_form_input',
						);

						// insert field postmeta
						foreach ($postmeta as $meta_key => $meta_val) {
							$wpdb->query($wpdb->prepare("INSERT INTO " . $wpdb->prefix . "postmeta (`post_id`, `meta_key`, `meta_value`) VALUES (%d, %s, %s)", $post_id, $meta_key, $meta_val));
						}
					}

					$aPostData = array(
						'post_author' => 1,
						'post_date' => current_time('mysql'),
						'post_date_gmt' => current_time('mysql'),
						'post_title' => $value->name,
						'post_content' => $value->name,
						'post_status' => 'publish',
						'comment_status' => 'closed',
						'ping_status' => 'closed',
						'post_parent' => 0,
						'guid' => get_bloginfo('url') . '/peepso_user_field/cpf/',
						'post_type' => 'peepso_user_field',
					);

					$post_id = wp_insert_post($aPostData);

					wp_update_post(array(
						'ID' => $post_id,
						'post_name' => $post_id
					));

					// default field postmeta
					$last_field_order = $value->field_order + $field_order;
					$postmeta = array(
						'order' => $last_field_order,
						'is_core' => 0,
						'_wp_old_slug' => 'cpf',
						'method' => '_render'
					);

					$this->add_memory('profile_field', $post_id, serialize(array(
						'value' => $value,
						'post_id' => $post_id,
						'postmeta' => $postmeta
					)));
				}

				$next_url = PeepsoMigrate::url('profile_field_value');
				if (isset($_GET['limit'])) {
					$next_url .= '&limit=' . $_GET['limit'];
				}
			}

			// append new option to core	
			// only if gender field, male option, female option is set
			if (!empty($gender_field) && !empty($gender_field_male) && !empty($gender_field_female)) {

				$sql = $wpdb->prepare("SELECT ID FROM " . $wpdb->prefix . "posts " .
						"WHERE post_name = %s AND post_type = %s AND post_status = %s", 'gender', 'peepso_user_field', 'publish');
				$peepso_gender_field = $wpdb->get_row($sql);

				$sql = $wpdb->prepare("SELECT A.name FROM " . $wpdb->prefix . "bp_xprofile_fields A " .
						"WHERE parent_id = %d AND type = %s AND id NOT IN ( %d, %d) ORDER BY id ASC", $gender_field, 'option', $gender_field_male, $gender_field_female);
				$all_custom_gender = $wpdb->get_results($sql);

				// append new option to core
				if (class_exists('PeepSoExtendedProfiles') && ($peepso_gender_field !== FALSE && $all_custom_gender !== FALSE)) {

					$list_of_gender = get_post_meta($peepso_gender_field->ID, 'select_options', true);

					for ($i = 1; $i <= count($all_custom_gender); $i++) {
						$gender_field_custom = get_transient('peepso_migrate_gender_field_custom' . ($i + 2));

						if (!empty($gender_field_custom)) {

							$sql = $wpdb->prepare("SELECT A.name FROM " . $wpdb->prefix . "bp_xprofile_fields A " .
									"WHERE id = %d AND type = %s AND id NOT IN ( %d, %d) ORDER BY id ASC", $gender_field_custom, 'option', $gender_field_male, $gender_field_female);
							$custom_field = $wpdb->get_row($sql);

							$list_of_gender['c' . $gender_field_custom] = $custom_field->name;
						}
					}

					update_post_meta($peepso_gender_field->ID, 'select_options', $list_of_gender);
				}
			}
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'profile_field',
			'counter' => 0,
			'general' => 7,
		));
	}

	function startMigrateProfileFieldValue() {
		global $wpdb;

		$next_url = PeepsoMigrate::url('profile_field_user_data');
		if (isset($_GET['limit'])) {
			$next_url .= '&limit=' . $_GET['limit'];
		}

		$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_peepso WHERE type = %s", 'profile_field');
		$all_profile_fields = $wpdb->get_results($sql);

		if (isset($all_profile_fields) && 0 < count($all_profile_fields)) {
			foreach ($all_profile_fields as $key => $item) {

				$new_id = unserialize($item->new_id);
				$value = $new_id['value'];
				$post_id = $new_id['post_id'];
				$postmeta = $new_id['postmeta'];

				// check if field is required
				if ($value->is_required == 1) {
					$postmeta['validation'] = serialize(array('required' => 1));
				}

				$profile_meta = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_meta WHERE object_id = %d", $value->id));
				if (isset($profile_meta) && count($profile_meta) > 0) {
					foreach ($profile_meta as $meta_key => $meta_value) {

						// check field visibility
						if ($meta_value->meta_key == 'default_visibility') {
							switch ($meta_value->meta_value) {
								case 'public' :
									$postmeta['default_acc'] = PeepSo::ACCESS_PUBLIC;
									break;
								case 'loggedin' :
									$postmeta['default_acc'] = PeepSo::ACCESS_MEMBERS;
									break;
								case 'friends' :
									if (class_exists('PeepSoFriendsPlugin')) {
										$postmeta['default_acc'] = PeepSoFriendsPlugin::ACCESS_FRIENDS;
									} else {
										$postmeta['default_acc'] = PeepSo::ACCESS_PUBLIC;
									}
									break;
								case 'adminsonly' :
									$postmeta['default_acc'] = PeepSo::ACCESS_PRIVATE;
									break;
							}
						}

						// check custom field visibility
						if ($meta_value->meta_key == 'allow_custom_visibility') {
							switch ($meta_value->meta_value) {
								case 'disabled' :
									$postmeta['user_disable_acc'] = 1;
									break;
								case 'allowed' :
									$postmeta['user_disable_acc'] = 0;
									break;
							}
						}
					}
				}

				// add additional postmeta into each field type
				switch ($value->type) {
					case 'selectbox':
					case 'multiselectbox' :
					case 'checkbox':
					case 'radio':
						$options = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_fields WHERE parent_id = %d ORDER BY option_order", $value->id));

						$select_options = array();
						$i = 1;
						if (isset($options) && count($options) > 0) {
							foreach ($options as $field_key => $field_value) {
								$select_options['option_' . $post_id . '_' . $i] = $field_value->name;
								$i++;
							}
						}

						$wpdb->query($wpdb->prepare("INSERT INTO " . $wpdb->prefix . "postmeta (post_id, meta_key, meta_value) VALUES (%d, %s, %s)", $post_id, 'select_options', serialize($select_options)));

						if ($value->type == 'selectbox') {
							$postmeta['class'] = 'selectsingle';
							$postmeta['method_form'] = '_render_form_select';
						} else if ($value->type == 'radio') {
							$postmeta['class'] = 'selectsingle';
							$postmeta['method_form'] = '_render_form_checklist';
						} else {
							$postmeta['class'] = 'selectmulti';
							$postmeta['method_form'] = '_render_form_checklist';
						}

						$this->add_memory('select_option', $post_id, serialize($select_options));
						break;
					case 'datebox' :
						$postmeta['class'] = 'textdate';
						$postmeta['method_form'] = '_render_form_input';
						break;
					case 'textbox' :
					case 'number' :
						$postmeta['class'] = 'text';
						$postmeta['method_form'] = '_render_form_input';
						break;
					case 'textarea' :
						$postmeta['class'] = 'text';
						$postmeta['method_form'] = '_render_form_textarea';
						break;
					case 'url' :
						$postmeta['class'] = 'texturl';
						$postmeta['method_form'] = '_render_form_input';
						break;
				}

				// insert field postmeta
				foreach ($postmeta as $meta_key => $meta_val) {
					$wpdb->query($wpdb->prepare("INSERT INTO " . $wpdb->prefix . "postmeta (`post_id`, `meta_key`, `meta_value`) VALUES (%d, %s, %s)", $post_id, $meta_key, $meta_val));
				}
			}
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'profile_field',
			'counter' => 0,
			'general' => 7,
		));
	}

	function startMigrateProfileFieldUserData() {
		global $wpdb;

		$next_url = PeepsoMigrate::url('profile_field&limit=' . $this->next_limit);
		$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_peepso WHERE type = %s", 'profile_field');
		$all_profile_fields = $wpdb->get_results($sql);

		if (isset($all_profile_fields) && 0 < count($all_profile_fields)) {
			foreach ($all_profile_fields as $key => $item) {

				$new_id = unserialize($item->new_id);
				$value = $new_id['value'];
				$post_id = $new_id['post_id'];
				$select_options = unserialize($this->get_memory('select_option', $post_id));

				// migrate user data
				$user_data = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_xprofile_data WHERE field_id = %d", $value->id));
				if (isset($user_data) && count($user_data) > 0) {
					foreach ($user_data as $user_key => $user_value) {
						switch ($value->type) {
							case 'multiselectbox' :
							case 'checkbox':
							case 'selectbox':
							case 'radio' :
								$selected_user_options = array();

								if ($value->type == 'selectbox' || $value->type == 'radio') {
									$selected_user_options_old = array($user_value->value);
								} else {
									$selected_user_options_old = unserialize($user_value->value);
								}
								foreach ($select_options as $option_key => $option_value) {
									if (in_array($option_value, $selected_user_options_old)) {
										$selected_user_options[] = $option_key;
									}
								}

								if ($value->type == 'radio' || $value->type == 'selectbox') {
									$selected_user_options = $selected_user_options[0];
								} else {
									$selected_user_options = serialize($selected_user_options);
								}
								break;
							case 'datebox' :
								$selected_user_options = str_replace(' 00:00:00', '', $user_value->value);
								break;
							default :
								$selected_user_options = $user_value->value;
								break;
						}

						$wpdb->query($wpdb->prepare("INSERT INTO " . $wpdb->prefix . "usermeta (user_id, meta_key, meta_value) VALUES (%d, %s, %s)", $user_value->user_id, 'peepso_user_field_' . $post_id, $selected_user_options));

						// update accessibility for custom profile fields
						$custom_acc = $this->_get_field_visibility($user_value->user_id, $value->id);
						update_user_meta($user_value->user_id, 'peepso_user_field_' . $post_id . '_acc', $custom_acc);
					}
				}
			}

			$this->counter = count($all_profile_fields);
		}

		$sql = $wpdb->prepare("delete from " . $wpdb->prefix . "bp_peepso where type = %s", 'profile_field');
		$wpdb->query($sql);

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'profile_field',
			'counter' => $this->counter,
			'general' => 7,
		));
	}

	function startMigrateGroups() {
		$next_url = PeepsoMigrate::url('group_member');

		if (PeepsoMigrate::plugin_check('group')) {
			register_post_type(PeepSoGroup::POST_TYPE);
			/*
			  NOT DONE YET :

			  MIGRATE GROUP PRIVACY
			  GENERATE GROUP URL

			 */
			global $wpdb;

			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_groups limit %d, %d", $this->limit, $this->per_page);
			$all_groups = $wpdb->get_results($sql);

			if (isset($all_groups) && 0 < count($all_groups)) {
				foreach ($all_groups as $key => $value) {

					$aPostData = array(
						'post_author' => $value->creator_id,
						'post_date' => $value->date_created,
						'post_date_gmt' => $value->date_created,
						'post_title' => $value->name,
						'post_content' => $value->description,
						'post_status' => 'publish',
						'comment_status' => 'closed',
						'ping_status' => 'closed',
						'post_parent' => 0,
						'post_type' => 'peepso-group',
					);

					$post_id = wp_insert_post($aPostData);

					$this->add_memory('group', $value->id, $post_id);

					$sql = "SELECT count(*) as total FROM " . $wpdb->prefix . "bp_groups_members where group_id = " . $value->id;
					$total_result = $wpdb->get_results($sql);
					$total_group_members = $total_result["0"]->total;
					add_post_meta($post_id, 'peepso_group_members_count', $total_group_members);
					
					$group_privacy = 0;
					if ($value->status == 'private' || $value->status == 'hidden') {
						$group_privacy = 1;
					}
					add_post_meta($post_id, 'peepso_group_privacy', $group_privacy);
				}

				$next_url = PeepsoMigrate::url('group&limit=' . $this->next_limit);
			}

			$this->counter = count($all_groups);
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'group',
			'counter' => $this->counter,
			'general' => 8,
		));
	}

	function startMigrateGroupMembers() {
		$next_url = PeepsoMigrate::url('group_avatar');

		if (PeepsoMigrate::plugin_check('group')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT A.*, B.creator_id FROM " . $wpdb->prefix . "bp_groups_members A LEFT JOIN " . $wpdb->prefix . "bp_groups B ON A.group_id = B.id limit %d, %d", $this->limit, $this->per_page);
			$all_group_members = $wpdb->get_results($sql);

			if (isset($all_group_members) && 0 < count($all_group_members)) {
				foreach ($all_group_members as $key => $value) {

					if ($value->user_id == $value->creator_id) {
						$user_status = 'member_owner';
					} else if ($value->is_admin == 1) {
						$user_status = 'member_admin';
					} else if ($value->is_mod == 1) {
						$user_status = 'member_moderator';
					} else if ($value->is_banned == 1) {
						$user_status = 'banned';
					} else if ($value->is_confirmed == 1) {
						$user_status = 'member';
					} else {
						$user_status = 'pending_user';
					}

					$wpdb->query($wpdb->prepare("INSERT INTO " . $wpdb->prefix . "peepso_group_members (gm_user_id, gm_group_id, gm_user_status, gm_joined) VALUES (%d, %d, %s, %d)", $value->user_id, $this->get_memory('group', $value->group_id), $user_status, $value->date_modified));
				}

				$next_url = PeepsoMigrate::url('group_member&limit=' . $this->next_limit);
			}

			$this->counter = count($all_group_members);
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'group_member',
			'counter' => $this->counter,
			'general' => 9,
		));
	}

	function startMigrateGroupAvatars() {
		$next_url = PeepsoMigrate::url('group_activity');

		if (PeepsoMigrate::plugin_check('group')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_groups limit %d, %d", $this->limit, $this->per_page);
			$all_groups = $wpdb->get_results($sql);

			@mkdir($this->peepso_dir);
			$target_dir = $this->peepso_dir . 'groups';
			@mkdir($target_dir);

			if (isset($all_groups) && 0 < count($all_groups)) {
				$path = get_home_path();

				foreach ($all_groups as $key => $group) {
					if (is_dir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "group-avatars" . DIRECTORY_SEPARATOR . $group->id)) {
						$buddy_avatars = scandir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "group-avatars" . DIRECTORY_SEPARATOR . $group->id);
						if (isset($buddy_avatars) && 0 < count($buddy_avatars)) {
							$use_custom_avatar = FALSE;

							foreach ($buddy_avatars as $poz => $file) {
								if ("." != $file && ".." != $file && "" != $file) {
									$new_group_id = $this->get_memory('group', $group->id);
									@mkdir($target_dir . DIRECTORY_SEPARATOR . $new_group_id);
									if (FALSE !== strpos($file, "bpfull")) {
										$file_array = explode(".", $file);
										$extension = $file_array[count($file_array) - 1];

										$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "group-avatars" . DIRECTORY_SEPARATOR . $group->id . DIRECTORY_SEPARATOR . $file;
										$destination1 = $target_dir . DIRECTORY_SEPARATOR . $new_group_id . DIRECTORY_SEPARATOR . 'avatar-full.jpg';
										$destination2 = $target_dir . DIRECTORY_SEPARATOR . $new_group_id . DIRECTORY_SEPARATOR . 'avatar-orig.jpg';

										if ($extension == 'png') {
											$this->_png2jpg($source, $destination1);
											$this->_png2jpg($source, $destination2);
										} else {
											copy($source, $destination1);
											copy($source, $destination2);
										}

										$use_custom_avatar = TRUE;
									} else if (FALSE !== strpos($file, "bpthumb")) {
										$file_array = explode(".", $file);
										$extension = $file_array[count($file_array) - 1];

										$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "group-avatars" . DIRECTORY_SEPARATOR . $group->id . DIRECTORY_SEPARATOR . $file;
										$destination = $target_dir . DIRECTORY_SEPARATOR . $new_group_id . DIRECTORY_SEPARATOR . 'avatar.jpg';

										if ($extension == 'png') {
											$this->_png2jpg($source, $destination);
										} else {
											copy($source, $destination);
										}

										$use_custom_avatar = TRUE;
									}
								}
							}

							if ($use_custom_avatar) {
								add_post_meta($new_group_id, 'group_avatar_custom', 1);
							}
						}
					}

					if (is_dir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "groups" . DIRECTORY_SEPARATOR . $group->id . DIRECTORY_SEPARATOR . "cover-image")) {
						$buddy_cover = scandir($path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "groups" . DIRECTORY_SEPARATOR . $group->id . DIRECTORY_SEPARATOR . "cover-image");

						if (isset($buddy_cover) && 0 < count($buddy_cover)) {
							foreach ($buddy_cover as $poz => $file) {
								if ("." != $file && ".." != $file && "" != $file) {
									$new_group_id = $this->get_memory('group', $group->id);
									@mkdir($target_dir . DIRECTORY_SEPARATOR . $new_group_id);
									if (FALSE !== strpos($file, "bp-cover-image")) {
										$file_array = explode(".", $file);
										$extension = $file_array[count($file_array) - 1];

										$source = $path . "wp-content" . DIRECTORY_SEPARATOR . "uploads" . DIRECTORY_SEPARATOR . "buddypress" . DIRECTORY_SEPARATOR . "groups" . DIRECTORY_SEPARATOR . $group->id . DIRECTORY_SEPARATOR . "cover-image" . DIRECTORY_SEPARATOR . $file;
										$destination = $target_dir . DIRECTORY_SEPARATOR . $new_group_id . DIRECTORY_SEPARATOR . 'cover.jpg';

										if ($extension == 'png') {
											$this->_png2jpg($source, $destination);
										} else {
											copy($source, $destination);
										}

										add_post_meta($new_group_id, 'group_cover_photo', PeepSo::get_peepso_uri() . 'groups/' . $new_group_id . '/cover.jpg');
									}
								}
							}
						}
					}
				}

				$next_url = PeepsoMigrate::url('group_avatar&limit=' . $this->next_limit);
			}

			$this->counter = count($all_groups);
		}

		echo json_encode(array(
			'url' => $next_url,
			'class' => 'group_avatar',
			'counter' => $this->counter,
			'general' => 10,
		));
	}

	function startMigrateGroupActivity() {
		$next_url = PeepsoMigrate::url('unpublish');

		if (PeepsoMigrate::plugin_check('group')) {
			global $wpdb;

			$sql = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "bp_activity where `component` = %s or `type` = %s limit %d, %d", 'groups', 'activity_comment', $this->limit, $this->per_page);
			$all_activities = $wpdb->get_results($sql);

			if (isset($all_activities) && 0 < count($all_activities)) {
				foreach ($all_activities as $key => $post) {

					// check if previous activity already migrated
					$check = $this->get_memory('activity', $post->id);
					if (!empty($check)) {
						continue;
					}

					$post_type = 'peepso-post';

					if (0 != $post->secondary_item_id) {
						$post_type = 'peepso-comment';
					}

					$post_content = strip_tags($post->content);

					if (empty($post_content)) {
						continue;
					}

					// convert tags
					if (class_exists('PeepSoTags')) {
						preg_match_all("/\@(\w+) /", $post_content, $matches);
						if (count($matches[1]) > 0) {
							foreach ($matches[1] as $username) {
								$user = get_user_by('slug', $username);

								if ($user) {
									$post_content = str_replace('@' . $username, '[peepso_tag id=' . $user->ID . ']' . $username . '[/peepso_tag]', $post_content);
								}
							}
						}
					}

					$aPostData = array(
						'post_author' => $post->user_id,
						'post_date' => $post->date_recorded,
						'post_date_gmt' => $post->date_recorded,
						'post_content' => $post_content,
						'post_title' => $post->user_id . "-" . $post->user_id . "-" . time(),
						'post_excerpt' => $post_content,
						'post_status' => 'publish',
						'comment_status' => 'open',
						'ping_status' => 'open',
						'post_password' => '',
						'post_name' => $post->user_id . "-" . $post->user_id . "-" . time(),
						'to_ping' => '',
						'pinged' => '',
						'post_modified' => $post->date_recorded,
						'post_modified_gmt' => $post->date_recorded,
						'post_content_filtered' => '',
						'post_parent' => 0,
						'guid' => get_site_url() . '/' . $post_type . '/' . $post->user_id . "-" . $post->user_id . "-" . time() . '/',
						'menu_order' => 0,
						'post_type' => $post_type,
						'post_mime_type' => '',
						'comment_count' => 0
					);

					$peepso_new_post_id = wp_insert_post($aPostData);
					$this->add_memory('activity', $post->id, $peepso_new_post_id);

					$group_id = $this->get_memory('group', $post->item_id);
					if (0 != $post->secondary_item_id) {
						$bp_peepso_parent_id = $this->get_memory('activity', $post->item_id);

						$sql = "SELECT A.id, A.item_id, @pv:=A.secondary_item_id AS parent FROM (select * from " . $wpdb->prefix . "bp_activity ORDER BY id DESC) A JOIN (SELECT @pv:=$post->secondary_item_id) tmp WHERE A.id=@pv";
						$parents = $wpdb->get_results($sql);

						if (count($parents) > 0) {
							foreach ($parents as $p) {
								if ($p->parent == $post->item_id) {
									$bp_peepso_parent_id = $this->get_memory('activity', $p->id);
								}

								if ($p->parent == 0) {
									$group_id = $this->get_memory('group', $p->item_id);
								}
							}
						}

						// if regular comment
						if ($post->item_id == $post->secondary_item_id) {
							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '1', '%d', '1', '1')", $post->user_id, intval($peepso_new_post_id), $bp_peepso_parent_id);
							$wpdb->query($sql);
						} else {
							$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '1', '%d', '1', '1')", $post->user_id, intval($peepso_new_post_id), $bp_peepso_parent_id);
							$wpdb->query($sql);
						}
					} else {
						$sql = $wpdb->prepare("insert into " . $wpdb->prefix . "peepso_activities (`act_owner_id`, `act_external_id`, `act_has_replies`, `act_comment_object_id`, `act_comment_module_id`, `act_module_id`) values ('%d', '%d', '0', '0', '0', '1')", $post->user_id, intval($peepso_new_post_id));
						$wpdb->query($sql);
					}

					add_post_meta($peepso_new_post_id, '_peepso_display_link_preview', 1);
					add_post_meta($peepso_new_post_id, 'peepso_group_id', $group_id);
				}

				$next_url = PeepsoMigrate::url('group_activity&limit=' . $this->next_limit);
			}

			$this->counter = count($all_activities);
		}


		echo json_encode(array(
			'url' => $next_url,
			'class' => 'group_activity',
			'counter' => $this->counter,
			'general' => 11,
		));
	}

	function unpublishBuddyPress() {
		global $wpdb;

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_activity]' where `post_type`='page' and `post_name`='activity'";
		$wpdb->query($sql);

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_members]' where `post_type`='page' and `post_name`='members'";
		$wpdb->query($sql);

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_profile]' where `post_type`='page' and `post_name`='profile'";
		$wpdb->query($sql);

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_register]' where `post_type`='page' and `post_name`='register'";
		$wpdb->query($sql);

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_recover]' where `post_type`='page' and `post_name`='password-recover'";
		$wpdb->query($sql);

		$sql = "update " . $wpdb->prefix . "posts set `post_content`='[peepso_groups]' where `post_type`='page' and `post_name`='groups'";
		$wpdb->query($sql);

		$sql = "DROP TABLE " . $wpdb->prefix . "bp_peepso";
		$wpdb->query($sql);

		$this->deactivatePlugins();

		echo json_encode(array(
			'finished' => 1,
			'counter' => $this->counter,
			'general' => 12,
		));
	}

	function deactivatePlugins() {
		global $wpdb;

		$sql = "SELECT `option_value` FROM " . $wpdb->prefix . "options where `option_name`='active_plugins'";
		$option_value = $wpdb->get_results($sql);

		if (isset($option_value) && isset($option_value["0"])) {
			$option_value = $option_value["0"]->option_value;
			$option_value = unserialize($option_value);

			if (isset($option_value) && 0 < count($option_value)) {
				$temp = array();
				foreach ($option_value as $key => $value) {
					if (FALSE === strpos(" " . $value, "buddypress") && FALSE === strpos(" " . $value, "peepsomigrate")) {
						$temp[] = $value;
					}
				}
				$option_value = $temp;
			}

			$sql = $wpdb->prepare("update " . $wpdb->prefix . "options set `option_value`='%s' where `option_name`='active_plugins'", serialize($option_value));
			$wpdb->query($sql);
		}
	}

	function deletePeepSoContent() {
		global $wpdb;

		$sql = "delete from " . $wpdb->prefix . "peepso_activities";
		$wpdb->query($sql);

		if (class_exists('PeepSoFriendsPlugin')) {
			$sql = "delete from " . $wpdb->prefix . "peepso_friends";
			$wpdb->query($sql);

			$sql = "delete from " . $wpdb->prefix . "peepso_friend_requests";
			$wpdb->query($sql);
		}

		if (class_exists('PeepSoMessagesPlugin')) {
			$sql = "delete from " . $wpdb->prefix . "peepso_message_participants";
			$wpdb->query($sql);

			$sql = "delete from " . $wpdb->prefix . "peepso_message_recipients";
			$wpdb->query($sql);
		}

		if (class_exists('PeepSoGroupsPlugin')) {
			$sql = "delete from " . $wpdb->prefix . "peepso_group_members";
			$wpdb->query($sql);
		}

		$sql = "delete from " . $wpdb->prefix . "peepso_notifications";
		$wpdb->query($sql);

		$sql = "delete from " . $wpdb->prefix . "peepso_users";
		$wpdb->query($sql);

		$sql = "delete from " . $wpdb->prefix . "posts where `post_type` like ('%peepso%')";
		$wpdb->query($sql);

		$sql = "delete from " . $wpdb->prefix . "postmeta where `meta_key` like '%peepso%'";
		$wpdb->query($sql);

		$sql = "delete from " . $wpdb->prefix . "usermeta where `meta_key` like '%peepso%'";
		$wpdb->query($sql);

		$sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'bp_peepso (
		  `id` bigint(20) NOT NULL AUTO_INCREMENT,
		  `type` TEXT NOT NULL,
		  `old_id` TEXT NOT NULL,
		  `new_id` TEXT NOT NULL,
		  PRIMARY KEY (`id`)
		) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;
		';
		$wpdb->query($sql);

		echo json_encode(array(
			'url' => PeepsoMigrate::url('profile')
		));
	}

	function _png2jpg($originalFile, $outputFile) {
		$image = imagecreatefrompng($originalFile);
		imagejpeg($image, $outputFile, 100);
		imagedestroy($image);
	}

	function _get_field_visibility($user_id, $field) {

		$visibility_levels = get_user_meta($user_id, 'bp_xprofile_visibility_levels', true);

		// check field visibility
		$default_acc = PeepSo::ACCESS_PUBLIC;
		if (!empty($visibility_levels) && is_array($visibility_levels)) {

			foreach ($visibility_levels as $key => $value) {
				if ($key == $field) {
					switch ($value) {
						case 'public' :
							$default_acc = PeepSo::ACCESS_PUBLIC;
							break;
						case 'loggedin' :
							$default_acc = PeepSo::ACCESS_MEMBERS;
							break;
						case 'friends' :
							if (class_exists('PeepSoFriendsPlugin')) {
								$default_acc = PeepSoFriendsPlugin::ACCESS_FRIENDS;
							} else {
								$default_acc = PeepSo::ACCESS_PUBLIC;
							}
							break;
						case 'adminsonly' :
							$default_acc = PeepSo::ACCESS_PRIVATE;
							break;
					}
				}
			}
		}

		return $default_acc;
	}

	function add_memory($type, $post_id, $peepso_new_post_id) {
		global $wpdb;

		$sql = $wpdb->prepare("INSERT INTO " . $wpdb->prefix . "bp_peepso (`type`, `old_id`, `new_id`) VALUES (%s, %s, %s)", $type, $post_id, $peepso_new_post_id);
		$wpdb->query($sql);
	}

	function get_memory($type, $old_id) {
		global $wpdb;

		$sql = $wpdb->prepare("SELECT `new_id` FROM " . $wpdb->prefix . "bp_peepso WHERE `type` = %s AND `old_id` = %s", $type, $old_id);
		$row = $wpdb->get_row($sql);
		if (isset($row->new_id)) {
			return $row->new_id;
		}
	}

}

?>